==> babel-plugin-typescript-to-proptypes-0.16.0.tgz: package/esm/index.js <==
/**
 * @copyright   2018-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
import { declare } from '@babel/helper-plugin-utils';
import { addDefault, addNamed } from '@babel/helper-module-imports';
import syntaxTypeScript from '@babel/plugin-syntax-typescript';
import { types as t } from '@babel/core';
import addToClass from './addToClass';
import addToFunctionOrVar from './addToFunctionOrVar';
import extractTypeProperties from './extractTypeProperties';
import upsertImport from './upsertImport';
const BABEL_VERSION = 7;
const MAX_DEPTH = 3;
const MAX_SIZE = 25;
const REACT_FC_NAMES = ['SFC', 'StatelessComponent', 'FC', 'FunctionComponent'];

function isNotTS(name) {
  return name.endsWith('.js') || name.endsWith('.jsx');
}

export default declare((api, options, root) => {
  api.assertVersion(BABEL_VERSION);
  return {
    inherits: syntaxTypeScript,

    manipulateOptions(opts, parserOptions) {
      parserOptions.plugins.push('jsx');
    },

    post() {
      this.state = null;
    },

    pre() {
      this.state = {
        airbnbPropTypes: {
          count: 0,
          forbidImport: '',
          hasImport: false,
          namedImports: []
        },
        componentTypes: {},
        filePath: '',
        options: {
          customPropTypeSuffixes: [],
          forbidExtraProps: false,
          maxDepth: MAX_DEPTH,
          maxSize: MAX_SIZE,
          typeCheck: false,
          ...options
        },
        propTypes: {
          count: 0,
          defaultImport: '',
          hasImport: false
        },
        reactImportedName: '',
        referenceTypes: {}
      };
    },

    visitor: {
      Program: {
        enter(programPath, {
          filename
        }) {
          const state = this.state;
          state.filePath = filename;

          if (isNotTS(filename)) {
            return;
          }

          programPath.node.body.forEach(node => {
            if (!t.isImportDeclaration(node)) {
              return;
            }

            if (node.source.value === 'prop-types') {
              const response = upsertImport(node, {
                checkForDefault: 'PropTypes'
              });
              state.propTypes.hasImport = true;
              state.propTypes.defaultImport = response.defaultImport;
            }

            if (node.source.value === 'airbnb-prop-types') {
              const response = upsertImport(node, {
                checkForNamed: 'forbidExtraProps'
              });
              state.airbnbPropTypes.hasImport = true;
              state.airbnbPropTypes.namedImports = response.namedImports;
              state.airbnbPropTypes.forbidImport = response.namedImport;
            }

            if (node.source.value === 'react') {
              const response = upsertImport(node, {
                checkForDefault: 'React'
              });
              state.reactImportedName = response.defaultImport;
            }
          });

          if (!state.propTypes.hasImport && state.reactImportedName) {
            state.propTypes.defaultImport = addDefault(programPath, 'prop-types', {
              nameHint: 'pt'
            }).name;
          }

          if (!state.airbnbPropTypes.hasImport && state.reactImportedName && options.forbidExtraProps) {
            state.airbnbPropTypes.forbidImport = addNamed(programPath, 'forbidExtraProps', 'airbnb-prop-types').name;
            state.airbnbPropTypes.count += 1;
          }

          if (!state.propTypes.defaultImport && !state.reactImportedName) {
            return;
          }

          programPath.traverse({
            CallExpression({
              node
            }) {
              const {
                namedImports
              } = state.airbnbPropTypes;

              if (options.forbidExtraProps && t.isIdentifier(node.callee) && namedImports.includes(node.callee.name)) {
                state.airbnbPropTypes.count += 1;
              }
            },

            'ClassDeclaration|ClassExpression': path => {
              const {
                node
              } = path;
              const valid = node.superTypeParameters && (t.isMemberExpression(node.superClass) && t.isIdentifier(node.superClass.object, {
                name: state.reactImportedName
              }) && (t.isIdentifier(node.superClass.property, {
                name: 'Component'
              }) || t.isIdentifier(node.superClass.property, {
                name: 'PureComponent'
              })) || state.reactImportedName && (t.isIdentifier(node.superClass, {
                name: 'Component'
              }) || t.isIdentifier(node.superClass, {
                name: 'PureComponent'
              })));

              if (valid) {
                addToClass(node, state);
              }
            },

            FunctionDeclaration(path) {
              const {
                node
              } = path;
              const param = node.params[0];
              const valid = param && state.reactImportedName && node.id.name.match(/^[A-Z]/u) && (t.isIdentifier(param) && param.typeAnnotation || t.isObjectPattern(param) && param.typeAnnotation);

              if (valid) {
                addToFunctionOrVar(path, node.id.name, state);
              }
            },

            Identifier({
              node
            }) {
              const {
                namedImports
              } = state.airbnbPropTypes;

              if (options.forbidExtraProps && namedImports.includes(node.name)) {
                state.airbnbPropTypes.count += 1;
              }
            },

            MemberExpression({
              node
            }) {
              if (t.isIdentifier(node.object, {
                name: state.propTypes.defaultImport
              })) {
                state.propTypes.count += 1;
              }
            },

            TSInterfaceDeclaration({
              node
            }) {
              state.componentTypes[node.id.name] = extractTypeProperties(node, state.componentTypes);
              state.referenceTypes[node.id.name] = node;
            },

            TSTypeAliasDeclaration({
              node
            }) {
              state.componentTypes[node.id.name] = extractTypeProperties(node, state.componentTypes);
              state.referenceTypes[node.id.name] = node;
            },

            VariableDeclaration(path) {
              const {
                node
              } = path;

              if (node.declarations.length === 0) {
                return;
              }

              const id = node.declarations[0].id;

              if (!id.typeAnnotation || !id.typeAnnotation.typeAnnotation) {
                return;
              }

              const type = id.typeAnnotation.typeAnnotation;
              const valid = t.isTSTypeReference(type) && type.typeParameters && type.typeParameters.params.length > 0 && (t.isTSQualifiedName(type.typeName) && t.isIdentifier(type.typeName.left, {
                name: state.reactImportedName
              }) && REACT_FC_NAMES.some(name => t.isIdentifier(type.typeName.right, {
                name
              })) || state.reactImportedName && REACT_FC_NAMES.some(name => t.isIdentifier(type.typeName, {
                name
              })));

              if (valid) {
                addToFunctionOrVar(path, id.name, state);
              }
            }

          });
        },

        exit(path, {
          filename
        }) {
          const state = this.state;

          if (isNotTS(filename)) {
            return;
          }

          path.get('body').forEach(bodyPath => {
            if (state.propTypes.count === 0 && t.isImportDeclaration(bodyPath.node) && bodyPath.node.source.value === 'prop-types') {
              bodyPath.remove();
            }
          });
        }

      }
    }
  };
});

==> babel-plugin-typescript-to-proptypes-0.17.0.tgz: package/esm/index.js <==
/**
 * @copyright   2018-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
import { declare } from '@babel/helper-plugin-utils';
import { addDefault, addNamed } from '@babel/helper-module-imports';
import syntaxTypeScript from '@babel/plugin-syntax-typescript';
import { types as t } from '@babel/core';
import addToClass from './addToClass';
import addToFunctionOrVar from './addToFunctionOrVar';
import extractTypeProperties from './extractTypeProperties';
import upsertImport from './upsertImport';
const BABEL_VERSION = 7;
const MAX_DEPTH = 3;
const MAX_SIZE = 25;
const REACT_FC_NAMES = ['SFC', 'StatelessComponent', 'FC', 'FunctionComponent'];

function isNotTS(name) {
  return name.endsWith('.js') || name.endsWith('.jsx');
}

function isComponentName(name) {
  return !!name.match(/^[A-Z]/u);
}

function isPropsParam(param) {
  return t.isIdentifier(param) && !!param.typeAnnotation || t.isObjectPattern(param) && !!param.typeAnnotation;
}

export default declare((api, options, root) => {
  api.assertVersion(BABEL_VERSION);
  return {
    inherits: syntaxTypeScript,

    manipulateOptions(opts, parserOptions) {
      parserOptions.plugins.push('jsx');
    },

    post() {
      this.state = null;
    },

    pre() {
      this.state = {
        airbnbPropTypes: {
          count: 0,
          forbidImport: '',
          hasImport: false,
          namedImports: []
        },
        componentTypes: {},
        filePath: '',
        options: {
          customPropTypeSuffixes: [],
          forbidExtraProps: false,
          maxDepth: MAX_DEPTH,
          maxSize: MAX_SIZE,
          typeCheck: false,
          ...options
        },
        propTypes: {
          count: 0,
          defaultImport: '',
          hasImport: false
        },
        reactImportedName: '',
        referenceTypes: {}
      };
    },

    visitor: {
      Program: {
        enter(programPath, {
          filename
        }) {
          const state = this.state;
          state.filePath = filename;

          if (isNotTS(filename)) {
            return;
          }

          programPath.node.body.forEach(node => {
            if (!t.isImportDeclaration(node)) {
              return;
            }

            if (node.source.value === 'prop-types') {
              const response = upsertImport(node, {
                checkForDefault: 'PropTypes'
              });
              state.propTypes.hasImport = true;
              state.propTypes.defaultImport = response.defaultImport;
            }

            if (node.source.value === 'airbnb-prop-types') {
              const response = upsertImport(node, {
                checkForNamed: 'forbidExtraProps'
              });
              state.airbnbPropTypes.hasImport = true;
              state.airbnbPropTypes.namedImports = response.namedImports;
              state.airbnbPropTypes.forbidImport = response.namedImport;
            }

            if (node.source.value === 'react') {
              const response = upsertImport(node, {
                checkForDefault: 'React'
              });
              state.reactImportedName = response.defaultImport;
            }
          });

          if (!state.propTypes.hasImport && state.reactImportedName) {
            state.propTypes.defaultImport = addDefault(programPath, 'prop-types', {
              nameHint: 'pt'
            }).name;
          }

          if (!state.airbnbPropTypes.hasImport && state.reactImportedName && options.forbidExtraProps) {
            state.airbnbPropTypes.forbidImport = addNamed(programPath, 'forbidExtraProps', 'airbnb-prop-types').name;
            state.airbnbPropTypes.count += 1;
          }

          if (!state.propTypes.defaultImport && !state.reactImportedName) {
            return;
          }

          const transformers = [];
          programPath.traverse({
            CallExpression({
              node
            }) {
              const {
                namedImports
              } = state.airbnbPropTypes;

              if (options.forbidExtraProps && t.isIdentifier(node.callee) && namedImports.includes(node.callee.name)) {
                state.airbnbPropTypes.count += 1;
              }
            },

            'ClassDeclaration|ClassExpression': path => {
              const {
                node
              } = path;
              const valid = node.superTypeParameters && (t.isMemberExpression(node.superClass) && t.isIdentifier(node.superClass.object, {
                name: state.reactImportedName
              }) && (t.isIdentifier(node.superClass.property, {
                name: 'Component'
              }) || t.isIdentifier(node.superClass.property, {
                name: 'PureComponent'
              })) || state.reactImportedName && (t.isIdentifier(node.superClass, {
                name: 'Component'
              }) || t.isIdentifier(node.superClass, {
                name: 'PureComponent'
              })));

              if (valid) {
                transformers.push(() => addToClass(node, state));
              }
            },

            FunctionDeclaration(path) {
              const {
                node
              } = path;
              const valid = !!state.reactImportedName && isComponentName(node.id.name) && isPropsParam(node.params[0]);

              if (valid) {
                transformers.push(() => addToFunctionOrVar(path, node.id.name, state));
              }
            },

            Identifier({
              node
            }) {
              const {
                namedImports
              } = state.airbnbPropTypes;

              if (options.forbidExtraProps && namedImports.includes(node.name)) {
                state.airbnbPropTypes.count += 1;
              }
            },

            MemberExpression({
              node
            }) {
              if (t.isIdentifier(node.object, {
                name: state.propTypes.defaultImport
              })) {
                state.propTypes.count += 1;
              }
            },

            TSInterfaceDeclaration({
              node
            }) {
              state.componentTypes[node.id.name] = extractTypeProperties(node, state.componentTypes);
              state.referenceTypes[node.id.name] = node;
            },

            TSTypeAliasDeclaration({
              node
            }) {
              state.componentTypes[node.id.name] = extractTypeProperties(node, state.componentTypes);
              state.referenceTypes[node.id.name] = node;
            },

            VariableDeclaration(path) {
              const {
                node
              } = path;

              if (node.declarations.length === 0) {
                return;
              }

              const decl = node.declarations[0];
              const id = decl.id;
              let valid = false;

              if (id.typeAnnotation && id.typeAnnotation.typeAnnotation) {
                const type = id.typeAnnotation.typeAnnotation;
                valid = t.isTSTypeReference(type) && !!type.typeParameters && type.typeParameters.params.length > 0 && (t.isTSQualifiedName(type.typeName) && t.isIdentifier(type.typeName.left, {
                  name: state.reactImportedName
                }) && REACT_FC_NAMES.some(name => t.isIdentifier(type.typeName.right, {
                  name
                })) || !!state.reactImportedName && REACT_FC_NAMES.some(name => t.isIdentifier(type.typeName, {
                  name
                })));
              } else if (t.isArrowFunctionExpression(decl.init)) {
                valid = !!state.reactImportedName && isComponentName(id.name) && isPropsParam(decl.init.params[0]);
              }

              if (valid) {
                transformers.push(() => addToFunctionOrVar(path, id.name, state));
              }
            }

          });
          transformers.forEach(transformer => {
            transformer();
          });
        },

        exit(path, {
          filename
        }) {
          const state = this.state;

          if (isNotTS(filename)) {
            return;
          }

          path.get('body').forEach(bodyPath => {
            if (state.propTypes.count === 0 && t.isImportDeclaration(bodyPath.node) && bodyPath.node.source.value === 'prop-types') {
              bodyPath.remove();
            }
          });
        }

      }
    }
  };
});
